// Pou itGlugu.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "Karel.h"
#include <stdio.h>
#include <gl/glew.h>
#include <GL/glut.h>
#include <gl/Gl.h>
#include <gl/glu.h>
#include <math.h>
#include <iostream>
#include <fstream>
using namespace std;

int pocet=0;
GLfloat Karel3D[10000];
GLfloat alpha= 90.0, beta= 0.0, dist=5.0;
int lastX=0, lastY=0;

GLuint VAOHandle[2];
void objectLoad(const int VAOH,
				const int numVertices,
				const float vertices[],
				const float normals[])
{
	//VBO
	GLuint VBOHandle[2];
	glGenBuffers(2, VBOHandle);

	GLuint positionBufferHandle= VBOHandle[0];
	GLuint normalBufferHandle= VBOHandle[1];

	if(vertices)
	{
		glBindBuffer(GL_ARRAY_BUFFER, positionBufferHandle);
		glBufferData(GL_ARRAY_BUFFER, 3*numVertices*sizeof(float), vertices, GL_STATIC_DRAW);
	}

	if(normals)
	{
		glBindBuffer(GL_ARRAY_BUFFER, normalBufferHandle);
		glBufferData(GL_ARRAY_BUFFER, 3*numVertices*sizeof(float), normals, GL_STATIC_DRAW);
	}
	

	//VAO
	glBindVertexArray(VAOH);

	// Enable Vertex Attribute Array
	if(vertices)	glEnableClientState(GL_VERTEX_ARRAY);
	if(normals)		glEnableClientState(GL_NORMAL_ARRAY);

	// Prirad index nula na positionBuffer
	glBindBuffer(GL_ARRAY_BUFFER, positionBufferHandle);
	glVertexPointer(3, GL_FLOAT, 0, (GLvoid *) NULL);

	// Prirad index jedna na normalBuffer
	glBindBuffer(GL_ARRAY_BUFFER, normalBufferHandle);
	glNormalPointer(GL_FLOAT,  0, (GLvoid *) NULL);

	glBindVertexArray(0);

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);

}


void init(void)
{
	ifstream in("Karel3D.raw");
	if(!in.is_open()){
		cout << "ERROR!!!" << endl;
		exit(0);
	}
	while(!in.eof())
		in >> Karel3D[pocet++];
	in.close();

	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClearDepth(1.0); //
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_NORMALIZE);

	GLfloat lightPosition[]= {1.0, 4.0, 1.0, -3.0};
	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);

	glGenVertexArrays(2, VAOHandle);
	objectLoad(VAOHandle[0], KarelNumVerts, KarelVerts, KarelNormals);
	objectLoad(VAOHandle[1], (GLsizei)(pocet/3), Karel3D, (GLfloat *) NULL);
}

void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	gluLookAt(0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0); //
	
	glTranslatef(0.0, 0.0, -dist);
	glRotatef(alpha, 0.0, 1.0, 0.0);
	glRotatef(beta, 1.0, 0.0, 0.0);

	//New
	glBindVertexArray(VAOHandle[0]);
	glDrawArrays(GL_TRIANGLES, 0, KarelNumVerts);

	////Old
	//glRotatef(-90, 0.0, 1.0, 0.0);
	//glBindVertexArray(VAOHandle[1]);
	//glDrawArrays(GL_TRIANGLES, 0, (GLsizei)(pocet/3));

	glutSwapBuffers();
	glFlush();
	
}

void reshape(int w, int h)
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	        //(Left, Right, Bottom, Top, Near, Far);
	glFrustum(-1.0, 1.0, -1.0, 1.0, 1.5, 3000.0);
	glMatrixMode(GL_MODELVIEW);
}


void myKeyboard(unsigned char key, int x, int y)
{
	switch(key)
	{
	case 'p':
		alpha+=5.0;
		glutPostRedisplay();
		break;
	case 'o':
		alpha-=5.0;
		glutPostRedisplay();
		break;
	case 'q':
		dist++;
		glutPostRedisplay();
		break;
	case 'a':
		dist--;
		glutPostRedisplay();
		break;
	case 27:
		exit(0);
		break;
	}
}

void myPassiveMotion(int x, int y){
	lastX=(GLfloat)x;
	lastY=(GLfloat)y;
}

void myMotion(int x, int y){
	alpha+= (GLfloat)x-lastX;
	beta+=(GLfloat)y-lastY;

	lastX=(GLfloat)x;
	lastY=(GLfloat)y;
	glutPostRedisplay();
}


int main(int argc,char **argv)
{	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(500, 500);
	glutInitWindowPosition(100, 100);
	glutCreateWindow(argv[0]);
	glewInit();

	
	
	init();
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(myKeyboard);
	glutMotionFunc(myMotion);
	glutPassiveMotionFunc(myPassiveMotion);
	glutMainLoop();
	return 0;
}